//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/DetectorEditor.cpp
//! @brief     Implements class DetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/DetectorEditor.h"
#include "GUI/Model/Detector/DetectorItem.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Support/Style/GroupBoxCollapser.h"
#include "GUI/View/Device/DetectorForm.h"
#include "GUI/View/Numeric/NumWidgetUtil.h"


DetectorEditor::DetectorEditor(QWidget* parent, GISASInstrumentItem* instrument)
    : QGroupBox("Detector parameters", parent)
    , m_instrument(instrument)
{
    ASSERT(instrument);
    m_formLayout = new QFormLayout(this);

    auto* collapser = GroupBoxCollapser::installIntoGroupBox(this);
    collapser->setExpanded(instrument->isExpandDetector());
    connect(collapser, &GroupBoxCollapser::toggled, this,
            [instrument](bool b) { instrument->setExpandDetector(b); });

    createDetectorWidgets();
}

void DetectorEditor::createDetectorWidgets()
{
    while (m_formLayout->rowCount() > 1)
        m_formLayout->removeRow(1);

    DetectorItem* detectorItem = m_instrument->detectorItem();
    auto* editor = new DetectorForm(this, detectorItem);
    m_formLayout->addRow(editor);
    connect(editor, &DetectorForm::dataChanged, this, &DetectorEditor::dataChanged);
}
