//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Numeric/ComboUtil.cpp
//! @brief     Implements functions in GUI::Util namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Numeric/ComboUtil.h"
#include "GUI/Model/Data/ComboProperty.h"
#include "GUI/View/Numeric/NumWidgetUtil.h"
#include <QFormLayout>
#include <QLabel>

using std::function;

QComboBox* GUI::Util::createComboBox(function<ComboProperty()> comboFunction,
                                     function<void(const QString&)> slot,
                                     QList<function<void()>>* updaters, QString tooltip,
                                     bool isScrollable)
{
    QComboBox* combo = new QComboBox;
    combo->addItems(comboFunction().values());
    combo->setMaxCount(comboFunction().values().size());
    combo->setToolTip(tooltip);
    combo->setCurrentText(comboFunction().currentValue());

    if (comboFunction().toolTips().size() == combo->count())
        for (int index = 0; index < combo->count(); index++)
            combo->setItemData(index, comboFunction().toolTips().at(index), Qt::ToolTipRole);

    if (!isScrollable)
        WheelEventEater::install(combo);

    QObject::connect(combo, &QComboBox::currentTextChanged, [=] { slot(combo->currentText()); });

    if (updaters)
        (*updaters) << [=]() {
            QSignalBlocker b(combo);
            combo->setCurrentText(comboFunction().currentValue());
        };

    return combo;
}

QComboBox* GUI::Util::createSafeComboBox(function<ComboProperty()> comboFunction,
                                         function<void(const QString&)> slot,
                                         QList<function<void()>>* updaters, QString tooltip)
{
    return createComboBox(comboFunction, slot, updaters, tooltip, false);
}

QComboBox* GUI::Util::createUnitsComboBox(const QStringList& list, function<QString()> currentUnits,
                                          function<void(const QString&)> slot,
                                          QList<function<void()>>* updaters, bool isScrollable)
{
    QComboBox* combo = new QComboBox;
    combo->addItems(list);
    combo->setMaxCount(list.size());
    combo->setCurrentText(currentUnits());

    if (!isScrollable)
        WheelEventEater::install(combo);

    QObject::connect(combo, &QComboBox::currentTextChanged, [=] { slot(combo->currentText()); });

    if (updaters)
        (*updaters) << [=]() {
            QSignalBlocker b(combo);
            combo->setCurrentText(currentUnits());
        };

    return combo;
}
